#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "param.h"
#include "str_utils.h"
#include "g2dot_msgs.h"
#include "g2dot_params.h"

char* _g2dot_program_name;
char* _g2dot_input_file_names;
char* _g2dot_output_file_name;
bool _g2dot_legend;

//---------------------------------------------------------------------------
bool g2dot_process_params(int argc, char **argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  g2dot_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _g2dot_program_name = namecopy(argv[0]);
  else
    _g2dot_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_g2dot_input_file_name]
    if (argv[i][0] != '-')
    {
      if (_g2dot_input_file_names == NULL)
        _g2dot_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _g2dot_input_file_names, argv[i]);
        delete [] _g2dot_input_file_names;
        _g2dot_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_g2dot_output_file_name] <=> --output [_g2dot_output_file_name]
    if (param_string(argc, argv, &i, "o", &_g2dot_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_g2dot_output_file_name))
    {
    }
    else
    // -l [_g2dot_legend] <=> --legend [_g2dot_legend]
    if (param_bool(argc, argv, &i, "l") ||
        param_bool(argc, argv, &i, "-legend"))
    {
      _g2dot_legend = true;
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _g2dot_help = g2dot_get_help(_g2dot_program_name);
      fprintf(stderr, _g2dot_help);
      delete [] _g2dot_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _g2dot_version = g2dot_get_version(_g2dot_program_name);
      fprintf(stderr, _g2dot_version);
      delete [] _g2dot_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  g2dot_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void g2dot_clear_params()
{
  delete [] _g2dot_program_name; _g2dot_program_name = NULL;
  delete [] _g2dot_input_file_names; _g2dot_input_file_names = NULL;
  delete [] _g2dot_output_file_name; _g2dot_output_file_name = NULL;
  _g2dot_legend = false;
}

//---------------------------------------------------------------------------
void g2dot_correct_params()
{
  if (_g2dot_program_name == NULL)
    _g2dot_program_name = namecopy(_G2DOT_PROGRAM_NAME);
  if (_g2dot_input_file_names == NULL)
    _g2dot_input_file_names = namecopy(_G2DOT_INPUT_FILE_NAMES);
  if (_g2dot_output_file_name == NULL)
    _g2dot_output_file_name = namecopy(_G2DOT_OUTPUT_FILE_NAME);
  // if no input file specified read it from standard input
  if (_g2dot_input_file_names == NULL)
    _g2dot_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_g2dot_output_file_name == NULL)
    _g2dot_output_file_name = namecopy(NAME_STDIO);
  if (_g2dot_legend < 0)
    _g2dot_legend = _G2DOT_LEGEND;

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namecount(_g2dot_input_file_names, NAME_STDIO);
  stdout_num += namecount(_g2dot_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}
