%{
#include "str_utils.h"
#include "stg_object.h"
#include "stg_protocol.h"
#include "protocol_y.h"

//#define DEBUG_LEX
#ifdef DEBUG_LEX
  #include <stdio.h>
  #define DEBUG_LEX_MESSAGE(msg) fprintf(stderr, msg)
#else
  #define DEBUG_LEX_MESSAGE(msg) // msg
#endif
%}

%option yylineno

NL                 [\n\r]
WS                 [\t\b ]
COMMENT            #[^\n\r]*
DIGIT              [0-9]
ALPHA              [a-zA-Z_]
ALPHANUM           [a-zA-Z_0-9]
INUMBER            {DIGIT}{DIGIT}*
RNUMBER1           {DIGIT}*\.{DIGIT}+
RNUMBER2           {DIGIT}+\.{DIGIT}*
RNUMBER            {RNUMBER1}|{RNUMBER2}
STRING             \"[^"]*\"
ID                 {ALPHA}+({DIGIT}*{ALPHA}*)*

%%

{WS}+              {
//  DEBUG_LEX_MESSAGE("WS\n");
  /* eat white spaces */
}

{COMMENT}         {
//  DEBUG_LEX_MESSAGE("COMMENT\n");
  /* eat comments */
}

{ID}               {
  DEBUG_LEX_MESSAGE("ID: "); DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  protocol_lval.string = namecopy(yytext);
  return PROTOCOL_ID;
}

.                  {
  DEBUG_LEX_MESSAGE(yytext); DEBUG_LEX_MESSAGE("\n");
  return(yytext[0]);
}

<<EOF>>            {
  DEBUG_LEX_MESSAGE("<EOF>\n");
  yyterminate();
  return 0;
}

%%

int yywrap()
{
  protocol_lineno = 0;
  // Free the buffer which remains un-freed due to a flex bug
  yy_delete_buffer(yy_current_buffer);
  yy_init = 1;
  return 1;
}
