%{
#include "messages.h"
#include "str_utils.h"
#include "stg_object.h"
#include "stg_protocol.h"
#include "protocol.h"

//#define DEBUG_YACC
#ifdef DEBUG_YACC
  #include <stdio.h>
  #define DEBUG_YACC_MESSAGE(msg) fprintf(stderr, msg)
#else
  #define DEBUG_YACC_MESSAGE(msg) // msg
#endif

ProtocolArray* protocol_data;
extern int protocol_lex();

void protocol_error(const char* message)
{
  Error("%s in line %d", message, protocol_lineno);
}

%}

%start protocols

%union
{
  char* string;
  Object* object;
  ObjectArray* objects;
  Protocol* protocol;
  ProtocolArray* protocols;
}

%token <string> PROTOCOL_ID
%type <object> object
%type <objects> objects
%type <protocol> protocol
%type <protocols> protocols

%%

protocols
  : protocol
    {
      DEBUG_YACC_MESSAGE("protocols : protocol\n");
      $$ = protocol_data;
      $$->add($1);
    } 
  | protocols ';' protocol
    {
      DEBUG_YACC_MESSAGE("protocols : protocols ';' protocol\n");
      $$ = $1;
      $$->add($3);
    }
;

protocol
  : PROTOCOL_ID '=' '{' objects '}'
    {
      DEBUG_YACC_MESSAGE("protocol : PROTOCOL_ID '=' '{' objects '}'\n");
      $$ = new Protocol($1);
      delete [] $1;
      $$->addObjects($4);
      delete $4;
    } 
;

objects
  : object
    {
      DEBUG_YACC_MESSAGE("objects : object\n");
      $$ = new ObjectArray();
      $$->add($1);
    }
  | objects ',' object
    {
      DEBUG_YACC_MESSAGE("objects : objects ',' object\n");
      $$ = $1;
      $$->add($3);
    }
;

object
  : PROTOCOL_ID
    {
      DEBUG_YACC_MESSAGE("object : PROTOCOL_ID\n");
      $$ = new Object($1);
      delete [] $1;
    }
;

%%
