#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "param.h"
#include "str_utils.h"
#include "protode_msgs.h"
#include "protode_params.h"

char* _protode_program_name;
char* _protode_input_file_names;
char* _protode_output_file_name;
char* _protode_protocol_signals;
char* _protode_protocols_extract;
char* _protode_protocols_optimise_a;
char* _protode_protocols_optimise_ab;
char* _protode_protocols_optimise_abc;
char* _protode_protocols_write;
int _protode_implicit_places;

//---------------------------------------------------------------------------
bool protode_process_params(int argc, char **argv)
{
  int i;
  int stdout_num = 1;
  bool result = true;

  protode_clear_params();
  char* program_name = strrchr(argv[0], '/');
  if (program_name == NULL)
    _protode_program_name = namecopy(argv[0]);
  else
    _protode_program_name = namecopy(&program_name[1]);

  if (argc < 2)
    i = 0;
  else
    i = 1;

  while (i > 0)
  {
    // [_protode_input_file_name]
    if (argv[i][0] != '-')
    {
      if (_protode_input_file_names == NULL)
        _protode_input_file_names = namecopy(argv[i]);
      else
      {
        char* input_file_name = name2pattern("%s,%s", _protode_input_file_names, argv[i]);
        delete [] _protode_input_file_names;
        _protode_input_file_names = input_file_name;
      }
      i++;
      if (i >= argc) i = 0;
      continue;
    }
    else
    // -o [_protode_output_file_name] <=> --output [_protode_output_file_name]
    if (param_string(argc, argv, &i, "o", &_protode_output_file_name) ||
        param_string(argc, argv, &i, "-output", &_protode_output_file_name))
    {
    }
    else
    // -p [_protode_protocol_signals] <=> --protocols [_protode_protocol_signals]
    if (param_string(argc, argv, &i, "p", &_protode_protocol_signals) ||
        param_string(argc, argv, &i, "-protocols", &_protode_protocol_signals))
    {
    }
    else
    // -ep [_protode_protocols_extract] <=> --extract-protocol [_protode_protocols_extract]
    if (param_string(argc, argv, &i, "ep", &_protode_protocols_extract) ||
        param_string(argc, argv, &i, "-extract-protocol", &_protode_protocols_extract))
    {
    }
    else
    // -o1 [_protode_protocols_optimise_a] <=> --optimise-a [_protode_protocols_optimise_a]
    if (param_string(argc, argv, &i, "o1", &_protode_protocols_optimise_a) ||
        param_string(argc, argv, &i, "-optimise-a", &_protode_protocols_optimise_a))
    {
    }
    else
    // -o2 [_protode_protocols_optimise_ab] <=> --optimise-a [_protode_protocols_optimise_ab]
    if (param_string(argc, argv, &i, "o2", &_protode_protocols_optimise_ab) ||
        param_string(argc, argv, &i, "-optimise-ab", &_protode_protocols_optimise_ab))
    {
    }
    else
    // -o3 [_protode_protocols_optimise_abc] <=> --optimise-abc [_protode_protocols_optimise_abc]
    if (param_string(argc, argv, &i, "o3", &_protode_protocols_optimise_abc) ||
        param_string(argc, argv, &i, "-optimise-abc", &_protode_protocols_optimise_abc))
    {
    }
    else
    // -wp [_protode_protocols_write] <=> -- [_protode_protocols_write]
    if (param_string(argc, argv, &i, "wp", &_protode_protocols_write) ||
        param_string(argc, argv, &i, "-write-protocol", &_protode_protocols_write))
    {
    }
    else
    // -ip [_protode_implicit_places] <=> --implicit-places [_protode_implicit_places]
    if (param_bool(argc, argv, &i, "ip") ||
        param_bool(argc, argv, &i, "-implicit-places"))
    {
      _protode_implicit_places = 1;
    }
    else
    // -h <=> --help
    if (param_bool(argc, argv, &i, "h") ||
        param_bool(argc, argv, &i, "-help"))
    {
      if (i != 0)
        Warning("<%s> must be the only flag",argv[i-1]);
      char* _protode_help = protode_get_help(_protode_program_name);
      fprintf(stderr, _protode_help);
      delete [] _protode_help;
      result = false;
    }
    else
    // -v <=> --version
    if (param_bool(argc, argv, &i, "v") ||
        param_bool(argc, argv, &i, "-vesion"))
    {
      if (i != 0)
        Warning("\"%s\" must be the only flag",argv[i-1]);
      char* _protode_version = protode_get_version(_protode_program_name);
      fprintf(stderr, _protode_version);
      delete [] _protode_version;
      result = false;
    }
    else
    // UNRECOGNIZED FLAG
    {
      Error("unrecognized flag \"%s\"", argv[i]);
      result = false;
    }
  }
  protode_correct_params();
  return result;
}

//---------------------------------------------------------------------------
void protode_clear_params()
{
  delete [] _protode_program_name; _protode_program_name = NULL;
  delete [] _protode_input_file_names; _protode_input_file_names = NULL;
  delete [] _protode_output_file_name; _protode_output_file_name = NULL;
  delete [] _protode_protocol_signals; _protode_protocol_signals = NULL;
  delete [] _protode_protocols_extract; _protode_protocols_extract = NULL;
  delete [] _protode_protocols_optimise_a; _protode_protocols_optimise_a = NULL;
  delete [] _protode_protocols_optimise_ab; _protode_protocols_optimise_ab = NULL;
  delete [] _protode_protocols_optimise_abc; _protode_protocols_optimise_abc = NULL;
  delete [] _protode_protocols_write; _protode_protocols_write = NULL;
  _protode_implicit_places = -1;
}

//---------------------------------------------------------------------------
void protode_correct_params()
{
  if (_protode_program_name == NULL)
    _protode_program_name = namecopy(_PROTODE_PROGRAM_NAME);
  if (_protode_input_file_names == NULL)
    _protode_input_file_names = namecopy(_PROTODE_INPUT_FILE_NAMES);
  if (_protode_output_file_name == NULL)
    _protode_output_file_name = namecopy(_PROTODE_OUTPUT_FILE_NAME);
  if (_protode_protocol_signals == NULL)
    _protode_protocol_signals = namecopy(_PROTODE_PROTOCOL_SIGNALS);
  if (_protode_protocols_extract == NULL)
    _protode_protocols_extract = namecopy(_PROTODE_PROTOCOLS_EXTRACT);
  if (_protode_protocols_optimise_a == NULL)
    _protode_protocols_optimise_a = namecopy(_PROTODE_PROTOCOLS_OPTIMISE_A);
  if (_protode_protocols_optimise_ab == NULL)
    _protode_protocols_optimise_ab = namecopy(_PROTODE_PROTOCOLS_OPTIMISE_AB);
  if (_protode_protocols_optimise_abc == NULL)
    _protode_protocols_optimise_abc = namecopy(_PROTODE_PROTOCOLS_OPTIMISE_ABC);
  if (_protode_protocols_write == NULL)
    _protode_protocols_write = namecopy(_PROTODE_PROTOCOLS_WRITE);
  // if no input file specified read it from standard input
  if (_protode_input_file_names == NULL)
    _protode_input_file_names = namecopy(NAME_STDIO);
  // if no output file specified write it to standard output
  if (_protode_output_file_name == NULL)
    _protode_output_file_name = namecopy(NAME_STDIO);
  if (_protode_implicit_places < 0)
    _protode_implicit_places = _PROTODE_IMPLICIT_PLACES;

  // count the number of files read from standard input and written to standard output
  int stdin_num = 0;
  int stdout_num = 0;

  stdin_num += namecount(_protode_input_file_names, NAME_STDIO);
  stdout_num += namecount(_protode_output_file_name, NAME_STDIO);

  if (stdin_num > 1)
    Warning("%d files will be read from standard input", stdin_num);
  if (stdout_num > 1)
    Warning("%d files will be written to standard output", stdout_num);
}
