#ifndef _STG_CLUSTER_H_
#define _STG_CLUSTER_H_

#include "stg_object.h"
#include "stg_transition.h"
#include "stg_place.h"

class Cluster;
typedef Array<Cluster, ObjectCompareName> ClusterArray;
typedef HashSet<Cluster, ObjectHashName, ObjectEqualName> ClusterHashSet;

//---------------------------------------------------------------------------
class Cluster : public Object
{
  public:
    enum Type 
    { 
      TYPE_UNDEFINED,  
      TYPE_INPUT, 
      TYPE_OUTPUT, 
      TYPE_INTERNAL, 
      TYPE_TRACKER 
    };

    Cluster(const char* _name = NULL, Type _type = TYPE_UNDEFINED, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Cluster(const Cluster& cluster);
    ~Cluster();

    void setType(Type _type);
    Type getType() const;
    
    int getTransitionCount() const;
    Transition* getTransition(const int transition_index) const;
    void addTransition(Transition* transition);
    void replaceTransition(Transition* transition, int position);
    void removeTransition(Transition* transition);
    int locateTransition(Transition* transition);

    int getPlaceCount() const;
    Place* getPlace(const int place_index) const;
    void addPlace(Place* place);
    void replacePlace(Place* place, int position);
    void removePlace(Place* place);
    int locatePlace(Place* place);

  protected:
    TransitionArray* transitions;
    PlaceArray* places;
    Type type;
};

//---------------------------------------------------------------------------
inline void Cluster::setType(Type _type)
{
  type = _type;
}

//---------------------------------------------------------------------------
inline Cluster::Type Cluster::getType() const
{
  return type;
}

//---------------------------------------------------------------------------
inline int Cluster::getTransitionCount() const
{
  return transitions->count();
}

//---------------------------------------------------------------------------
inline Transition* Cluster::getTransition(int transition_index) const
{
  return (*transitions)[transition_index];
}

//---------------------------------------------------------------------------
inline void Cluster::addTransition(Transition* transition)
{
  transitions->add(transition);
}

//---------------------------------------------------------------------------
inline void Cluster::replaceTransition(Transition* transition, int position)
{
  (*transitions)[position] = transition;
}

//---------------------------------------------------------------------------
inline void Cluster::removeTransition(Transition* transition)
{
  transitions->remove(transition, false);
}

//---------------------------------------------------------------------------
inline int Cluster::locateTransition(Transition* transition)
{
  return transitions->locate(transition);
}

//---------------------------------------------------------------------------
inline int Cluster::getPlaceCount() const
{
  return places->count();
}

//---------------------------------------------------------------------------
inline Place* Cluster::getPlace(int place_index) const
{
  return (*places)[place_index];
}

//---------------------------------------------------------------------------
inline void Cluster::addPlace(Place* place)
{
  places->add(place);
}

//---------------------------------------------------------------------------
inline void Cluster::replacePlace(Place* place, int position)
{
  (*places)[position] = place;
}

//---------------------------------------------------------------------------
inline void Cluster::removePlace(Place* place)
{
  places->remove(place, false);
}

//---------------------------------------------------------------------------
inline int Cluster::locatePlace(Place* place)
{
  return places->locate(place);
}

#endif
