#include <stdio.h>
#include "messages.h"
#include "stg_object.h"
#include "stg_transition.h"
#include "stg_place.h"
#include "stg_cluster.h"
#include "stg_notation.h"
#include "stg_graph_cluster.h"
#include "stg_graph_cluster_dot.h"

//---------------------------------------------------------------------------
const char* DOT_KEYWORD_DIGRAPH         = "digraph";
const char* DOT_KEYWORD_SUBGRAPH        = "subgraph";
const char* DOT_KEYWORD_CLUSTER         = "cluster";
const char* DOT_KEYWORD_STATEMENT_BEGIN = "{";
const char* DOT_KEYWORD_STATEMENT_END   = "}";
const char* DOT_KEYWORD_ATTRIB_BEGIN    = "[";
const char* DOT_KEYWORD_ATTRIB_END      = "]";
const char* DOT_KEYWORD_NODE            = "node";
const char* DOT_KEYWORD_EDGE            = "edge";
const char* DOT_KEYWORD_ARC             = "->";
const char* DOT_KEYWORD_LABEL           = "label";
const char* DOT_KEYWORD_COLOR           = "color";
const char* DOT_KEYWORD_CENTER          = "center";
const char* DOT_KEYWORD_SIZE            = "size";
const char* DOT_KEYWORD_MARGIN          = "margin";
const char* DOT_KEYWORD_RATIO           = "ratio";

const char* DOT_ATTRIB_ARC              =  "arrowhead=normal, arrowtail=none, weight=100, color=black, style=solid";
const char* DOT_ATTRIB_READARC          =  "arrowhead=none, arrowtail=none, weight=1, color=gray, style=solid";
const char* DOT_ATTRIB_PLACE_EMPTY      =  "shape=circle, fontsize=11, fontcolor=black, height=0.4, width=0.4, fixedsize=true, color=black";
const char* DOT_ATTRIB_PLACE_EMPTY_INPUT=  "shape=circle, fontsize=11, fontcolor=red, height=0.4, width=0.4, fixedsize=true, color=black";
const char* DOT_ATTRIB_PLACE_EMPTY_OUTPUT =  "shape=circle, fontsize=11, fontcolor=blue, height=0.4, width=0.4, fixedsize=true, color=black";
const char* DOT_ATTRIB_PLACE_MARKED     =  "shape=circle, fontsize=11, fontcolor=grey, height=0.3, width=0.3, fixedsize=true, color=black, peripheries=2, style=filled";
const char* DOT_ATTRIB_PLACE_MARKED_INPUT =  "shape=circle, fontsize=11, fontcolor=red, height=0.3, width=0.3, fixedsize=true, color=black, peripheries=2, style=filled";
const char* DOT_ATTRIB_PLACE_MARKED_OUTPUT =  "shape=circle, fontsize=11, fontcolor=blue, height=0.3, width=0.3, fixedsize=true, color=black, peripheries=2, style=filled";
const char* DOT_ATTRIB_PLACE_REDUNDANT_EMPTY =  "shape=circle, fontsize=11, fontcolor=black, height=0.4, width=0.4, fixedsize=true, color=gray";
const char* DOT_ATTRIB_PLACE_REDUNDANT_MARKED =  "shape=circle, fontsize=11, fontcolor=gray, height=0.3, width=0.3, fixedsize=true, color=gray, fillcolor=black, peripheries=2, style=filled";
const char* DOT_ATTRIB_PLACE_COPY       =  "shape=plaintext, fontsize=11, fontcolor=black, height=0.2, width=0.4, fixedsize=true";
const char* DOT_ATTRIB_PLACE_COPY_INPUT =  "shape=plaintext, fontsize=11, fontcolor=red, height=0.2, width=0.4, fixedsize=true";
const char* DOT_ATTRIB_PLACE_COPY_OUTPUT=  "shape=plaintext, fontsize=11, fontcolor=blue, height=0.2, width=0.4, fixedsize=true";
const char* DOT_ATTRIB_TRANSITION       =  "shape=box, fontsize=11, fontcolor=black, height=0.3, width=0.8, fixedsize=true, color=black, regular=0";
const char* DOT_ATTRIB_TRANSITION_INPUT =  "shape=box, fontsize=11, fontcolor=red,   height=0.3, width=0.8, fixedsize=true, color=black";
const char* DOT_ATTRIB_TRANSITION_OUTPUT=  "shape=box, fontsize=11, fontcolor=blue,  height=0.3, width=0.8, fixedsize=true, color=black";
 
const char* DOT_COLOR_WHITE             = "white";
const char* DOT_LABEL_MODEL             = "MODEL:";
const char* DOT_LABEL_INPUTS            = "INPUTS:";
const char* DOT_LABEL_OUTPUTS           = "OUTPUTS:";
const char* DOT_LABEL_INTERNAL          = "INTERNAL:";
const char* DOT_LABEL_DUMMY             = "DUMMY:";
const char* DOT_LABEL_DELIMITER         = ", ";
const char* DOT_LABEL_EMPTY             = "\"\"";
const char* DOT_LABEL_EXPOSED_PLACE_PATTERN_LOW = "%s=0";
const char* DOT_LABEL_EXPOSED_PLACE_PATTERN_HIGH = "%s=1";
const char* DOT_LABEL_EXPOSED_DUMMY_PATTERN = "(%s)";
const char* DOT_VARIABLE_TRANSITION_PATTERN_RISING = "%s_plus";
const char* DOT_VARIABLE_TRANSITION_PATTERN_FALLING = "%s_minus";
const char* DOT_VARIABLE_TRANSITION_PATTERN_INDEX = "%s_%d";

//---------------------------------------------------------------------------
GraphClusterDot::GraphClusterDot(const char* _name, Tag _tag, void* _data)
: GraphCluster(_name, _tag, _data)
{
}

//---------------------------------------------------------------------------
GraphClusterDot::GraphClusterDot(const GraphClusterDot& graph_cluster_dot)
: GraphCluster(graph_cluster_dot)
{
}

//---------------------------------------------------------------------------
GraphClusterDot::~GraphClusterDot()
{
}

//---------------------------------------------------------------------------
char* GraphClusterDot::genDotModelLabel(const char* model_label)
{
  char* label = NULL;
  if (model_label)
    label = namecopy(model_label);
  else
    label = namecopy(" ");
    
  for (int i = 0; i < strlen(label); i++)
  {
    if ( (label[i] == ' ') || (label[i] == '\t') || (label[i] == '\n') || (label[i] == '\r') )
      label[i] = SYMBOL_UNDERSCORE;
  }
  return label;
}


//---------------------------------------------------------------------------
char* GraphClusterDot::genDotSignalLabel(Signal* signal)
{
  char* label = NULL;
  if (signal)
  {
    const char* signal_name = signal->getName();
    if (signal->getType() == Signal::TYPE_DUMMY)
    {
      int len_protocol = 0;
      int len_protocol_prefix = strlen(NOTATION_PROTOCOL_DUMMY_PREFIX);
      int len_protocol_suffix = strlen(NOTATION_PROTOCOL_DUMMY_SUFFIX);
      if ( strncmp(signal_name, NOTATION_PROTOCOL_DUMMY_PREFIX, len_protocol_prefix) == 0 )
      {
        char* str_tmp = strstr(&signal_name[len_protocol_prefix], NOTATION_PROTOCOL_DUMMY_SUFFIX);
        if (str_tmp) 
          len_protocol = (str_tmp - signal_name) + len_protocol_suffix;
      }
      char* signal_name_tmp = namecopy( &signal_name[len_protocol] );
  
      char* str_plus = strstr(signal_name_tmp, NOTATION_EXPOSED_DUMMY_SUFFIX_RISING);
      char* str_minus = strstr(signal_name_tmp, NOTATION_EXPOSED_DUMMY_SUFFIX_FALLING);
      if ( !str_plus && !str_minus)
        label = namecopy(signal_name_tmp);
      else
      {
        label = name2pattern(DOT_LABEL_EXPOSED_DUMMY_PATTERN, signal_name_tmp);
        if (str_plus)
        {
          int i = str_plus - signal_name_tmp + 1;
          label[i] = TRANSITION_SYMBOL_RISING;
          strcpy( &label[i + 1], &label[i + strlen(NOTATION_EXPOSED_DUMMY_SUFFIX_RISING)] );
        }
        else
        {
          int i = str_minus - signal_name_tmp + 1;
          label[i] = TRANSITION_SYMBOL_FALLING;
          strcpy( &label[i + 1], &label[i + strlen(NOTATION_EXPOSED_DUMMY_SUFFIX_FALLING)] );
        }
      }
      delete [] signal_name_tmp;
    }
    if (label == NULL) label = namecopy(signal_name);
  }
  return label;
}

//---------------------------------------------------------------------------
char* GraphClusterDot::genDotTransitionLabel(Transition* transition)
{
  char* label = NULL;
  Signal* signal = findSignal(transition->getSignalName());
  if ( signal && (signal->getType() == Signal::TYPE_DUMMY) )
    label = genDotSignalLabel(signal);
  else
    label = namecopy(transition->getName());
  char* pos_index = strchr(label, TRANSITION_SYMBOL_INDEX);
  if (pos_index)
    pos_index[0] = '\0';
  return label;
}

//---------------------------------------------------------------------------
char* GraphClusterDot::genDotPlaceLabel(Place* place)
{
  char* label = NULL;
  char* label_tmp = NULL;

  // redundant place
  const char* place_name = place->getName();
  int len = strlen(place_name);
  int len_redundant = strlen(NOTATION_REDUNDANT_PLACE_PREFIX);
  if ( (len >= len_redundant)
    && ( strncmp(place_name, NOTATION_REDUNDANT_PLACE_PREFIX, len_redundant) == 0 ) )
  {
    label_tmp = namecopy(&place_name[len_redundant]);
    len = strlen(label_tmp);
  }
  else
    label_tmp = namecopy(place_name);

  // place=0 or place=1
  int len_high = strlen(NOTATION_EXPOSED_PLACE_SUFFIX_HIGH);
  int len_low = strlen(NOTATION_EXPOSED_PLACE_SUFFIX_LOW);

  if (len >= len_high)
  {
    int pos_high = len-len_high;
    if ( strncmp(&label_tmp[pos_high], NOTATION_EXPOSED_PLACE_SUFFIX_HIGH, len_high) == 0 )
    {
      label_tmp[pos_high] = '\0';
      label = name2pattern(DOT_LABEL_EXPOSED_PLACE_PATTERN_HIGH, label_tmp);
    }
    else
    if (len >= len_low)
    {
      int pos_low = len-len_low;
      if ( strncmp(&label_tmp[pos_low], NOTATION_EXPOSED_PLACE_SUFFIX_LOW, len_low) == 0 )
      {
        label_tmp[pos_low] = '\0';
        label = name2pattern(DOT_LABEL_EXPOSED_PLACE_PATTERN_LOW, label_tmp);
      }
    }
  }
  if (label == NULL)
    label = namecopy(label_tmp);
  delete [] label_tmp;
  return label;
}

//---------------------------------------------------------------------------
char* GraphClusterDot::genDotTransitionVariable(Transition* transition)
{
  char* var = NULL;
  char* var_tmp = NULL;
  Signal* signal = findSignal(transition->getSignalName());
  if ( signal && (signal->getType() == Signal::TYPE_DUMMY) )
    var_tmp = namecopy(transition->getSignalName());
  else
  {
    if (transition->getPolarity() == Transition::POLARITY_RISING)
      var_tmp = name2pattern(DOT_VARIABLE_TRANSITION_PATTERN_RISING, transition->getSignalName());
    else
    if (transition->getPolarity() == Transition::POLARITY_FALLING)
      var_tmp = name2pattern(DOT_VARIABLE_TRANSITION_PATTERN_FALLING, transition->getSignalName());
  }
  int index = transition->getIndex();
  if (index < 0)
    var = var_tmp;
  else
  {
    var = name2pattern(DOT_VARIABLE_TRANSITION_PATTERN_INDEX, var_tmp, index);
    delete [] var_tmp;
  }
  return var;
}

//---------------------------------------------------------------------------
char* GraphClusterDot::genDotPlaceVariable(Place* place)
{
  return namecopy(place->getName());
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotSignals(FILE* file, Signal::Type type, const char* label)
{
  bool label_printed = false;
  for (int signal_index = 0; signal_index < signals->count(); signal_index++)
  {
    Signal* signal_cur = (*signals)[signal_index];
    if (signal_cur->getType() == type)
    {
      if (!label_printed)
      {
        fprintf(file, "\\n%s ", label);
        label_printed = true;
      }
      else 
        fprintf(file, DOT_LABEL_DELIMITER);
      char* signal_label = genDotSignalLabel(signal_cur);
      fprintf(file, signal_label);
      delete [] signal_label;
    }
  }
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotLegend(FILE* file)
{
  char* model_tmp = NULL;
  if (model)
    model_tmp = namecopy(model);
  else
    model_tmp = namecopy("");
  fprintf(file, "  %s=\"%s %s", DOT_KEYWORD_LABEL, DOT_LABEL_MODEL, model_tmp);
  delete [] model_tmp;
  writeDotSignals(file, Signal::TYPE_INPUT, DOT_LABEL_INPUTS);
  writeDotSignals(file, Signal::TYPE_OUTPUT, DOT_LABEL_OUTPUTS);
  writeDotSignals(file, Signal::TYPE_INTERNAL, DOT_LABEL_INTERNAL);
//  writeDotSignals(file, Signal::TYPE_DUMMY, DOT_LABEL_DUMMY);
  fprintf(file, "\"\n");
  fprintf(file, "  %s=%s;\n", DOT_KEYWORD_COLOR, DOT_COLOR_WHITE);
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotClusterProducingArcs(FILE* file, int cluster_index)
{
  bool attrib_printed = false;
  Cluster* cluster_cur = (*clusters)[cluster_index];
  for (int transition_index = 0; transition_index < cluster_cur->getTransitionCount(); transition_index++)
  {
    Transition* transition_cur = cluster_cur->getTransition(transition_index);
    if (transition_cur->getPostsetCount() > 0)
    {
      if ( !attrib_printed )
      {
        fprintf(file, "\n");
        fprintf (file, "    %s %s%s%s;\n", DOT_KEYWORD_EDGE, DOT_KEYWORD_ATTRIB_BEGIN, DOT_ATTRIB_ARC, DOT_KEYWORD_ATTRIB_END);
        attrib_printed = true;
      }
      char* transition_var = genDotTransitionVariable(transition_cur);
      for (int place_index = 0; place_index < transition_cur->getPostsetCount(); place_index++)
      {
        Place* place_cur = transition_cur->getPostset(place_index);
        char* place_var = genDotPlaceVariable(place_cur);
        fprintf(file, "    %s %s %s;\n", transition_var, DOT_KEYWORD_ARC, place_var);
        delete [] place_var;
      }
      delete [] transition_var;
    }
  }
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotClusterConsumingArcs(FILE* file, int cluster_index)
{
  bool attrib_printed = false;
  Cluster* cluster_cur = (*clusters)[cluster_index];
  for (int place_index = 0; place_index < cluster_cur->getPlaceCount(); place_index++)
  {
    Place* place_cur = cluster_cur->getPlace(place_index);
    if (place_cur->getPostsetCount() > 0)
    {
      if ( !attrib_printed )
      {
        fprintf(file, "\n");
        fprintf (file, "    %s %s%s%s;\n", DOT_KEYWORD_EDGE, DOT_KEYWORD_ATTRIB_BEGIN, DOT_ATTRIB_ARC, DOT_KEYWORD_ATTRIB_END);
        attrib_printed = true;
      }
      char* place_var = genDotPlaceVariable(place_cur);
      for (int transition_index = 0; transition_index < place_cur->getPostsetCount(); transition_index++)
      {
        Transition* transition_cur = place_cur->getPostset(transition_index);
        char* transition_var = genDotTransitionVariable(transition_cur);
        fprintf(file, "    %s %s %s;\n", place_var, DOT_KEYWORD_ARC, transition_var);
        delete [] transition_var;
      }
      delete [] place_var;
    }
  }
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotClusterReadArcs(FILE* file, int cluster_index)
{
  bool attrib_printed = false;
  Cluster* cluster_cur = (*clusters)[cluster_index];
  int item_index = 0;
  for (int transition_index = 0; transition_index < cluster_cur->getTransitionCount(); transition_index++)
  {
    Transition* transition_cur = cluster_cur->getTransition(transition_index);
    if (transition_cur->getReadsetCount() > 0)
    {
      if ( !attrib_printed )
      {
        fprintf(file, "\n");
        fprintf (file, "    %s %s%s%s;\n", DOT_KEYWORD_EDGE, DOT_KEYWORD_ATTRIB_BEGIN, DOT_ATTRIB_READARC, DOT_KEYWORD_ATTRIB_END);
        attrib_printed = true;
      }
      char* transition_var = genDotTransitionVariable(transition_cur);
      for (int place_index = 0; place_index < transition_cur->getReadsetCount(); place_index++)
      {
        Place* place_cur = transition_cur->getReadset(place_index);
        char* place_var = genDotPlaceVariable(place_cur);
        char* place_print = genDotPlaceLabel(place_cur);
        fprintf(file, "    %s_c%.2d_i%.2d %s %s;\n", place_var, cluster_index, item_index, 
          DOT_KEYWORD_ARC, transition_var);
        Cluster* cluster_ctrl = findCluster(place_cur);
        if (cluster_ctrl)
        {
          if (cluster_ctrl->getType() == Cluster::TYPE_INPUT)
            fprintf(file, "    %s_c%.2d_i%.2d %s%s=\"%s\", %s%s;\n", place_var, 
              cluster_index, item_index, DOT_KEYWORD_ATTRIB_BEGIN, DOT_KEYWORD_LABEL, 
              place_print, DOT_ATTRIB_PLACE_COPY_INPUT, DOT_KEYWORD_ATTRIB_END);
          else
          if (cluster_ctrl->getType() == Cluster::TYPE_OUTPUT)
            fprintf(file, "    %s_c%.2d_i%.2d %s%s=\"%s\", %s%s;\n", place_var, 
              cluster_index, item_index, DOT_KEYWORD_ATTRIB_BEGIN, DOT_KEYWORD_LABEL, 
              place_print, DOT_ATTRIB_PLACE_COPY_OUTPUT, DOT_KEYWORD_ATTRIB_END);
          else
            fprintf(file, "    %s_c%.2d_i%.2d %s%s=\"%s\", %s%s;\n", place_var, 
              cluster_index, item_index, DOT_KEYWORD_ATTRIB_BEGIN, DOT_KEYWORD_LABEL, 
              place_print, DOT_ATTRIB_PLACE_COPY, DOT_KEYWORD_ATTRIB_END);
        }
        else
          fprintf(file, "    %s_c%.2d_i%.2d %s%s=\"%s\", %s%s;\n", place_var, 
            cluster_index, item_index, DOT_KEYWORD_ATTRIB_BEGIN, DOT_KEYWORD_LABEL, 
            place_print, DOT_ATTRIB_PLACE_COPY_INPUT, DOT_KEYWORD_ATTRIB_END);
        item_index++;
        delete [] place_var;
        delete [] place_print;
      }
      delete [] transition_var;
    }
  }
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotClusterTransitions(FILE* file, int cluster_index)
{
  Cluster* cluster_cur = (*clusters)[cluster_index];
  for (int transition_index = 0; transition_index < cluster_cur->getTransitionCount(); transition_index++)
  {
    Transition* transition_cur = cluster_cur->getTransition(transition_index);
    char* transition_var = genDotTransitionVariable(transition_cur);
    char* transition_label = genDotTransitionLabel(transition_cur);
    Signal* signal_cur = findSignal(transition_cur->getSignalName());
    if (signal_cur)
    {
      if (signal_cur->getType() == Signal::TYPE_INPUT)
        fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", transition_var, DOT_KEYWORD_ATTRIB_BEGIN, 
          DOT_KEYWORD_LABEL, transition_label, DOT_ATTRIB_TRANSITION_INPUT, DOT_KEYWORD_ATTRIB_END);
      else
      if (signal_cur->getType() == Signal::TYPE_OUTPUT)
        fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", transition_var, DOT_KEYWORD_ATTRIB_BEGIN, 
          DOT_KEYWORD_LABEL, transition_label, DOT_ATTRIB_TRANSITION_OUTPUT, DOT_KEYWORD_ATTRIB_END);
      else
        fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", transition_var, DOT_KEYWORD_ATTRIB_BEGIN, 
          DOT_KEYWORD_LABEL, transition_label, DOT_ATTRIB_TRANSITION, DOT_KEYWORD_ATTRIB_END);
    }
    delete [] transition_var;
    delete [] transition_label;
  }
}
 
//---------------------------------------------------------------------------
void GraphClusterDot::writeDotClusterPlaces(FILE* file, int cluster_index)
{
  Cluster* cluster_cur = (*clusters)[cluster_index];
  for (int place_index = 0; place_index < cluster_cur->getPlaceCount(); place_index++)
  {
    Place* place_cur = cluster_cur->getPlace(place_index);
    char* place_var = genDotPlaceVariable(place_cur);
    char* place_label = genDotPlaceLabel(place_cur);
    int len = strlen(NOTATION_REDUNDANT_PLACE_PREFIX);
    if (place_cur->getMarking() == 0)
    {
      if ( strncmp(place_cur->getName(), NOTATION_REDUNDANT_PLACE_PREFIX, len) == 0 )
        fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN,
          DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_REDUNDANT_EMPTY, DOT_KEYWORD_ATTRIB_END);
      else
      {
        if (cluster_cur->getType() == Cluster::TYPE_INPUT)
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
            DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_EMPTY_INPUT, DOT_KEYWORD_ATTRIB_END);
        else
        if (cluster_cur->getType() == Cluster::TYPE_OUTPUT)
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
            DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_EMPTY_OUTPUT, DOT_KEYWORD_ATTRIB_END);
        else
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
            DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_EMPTY, DOT_KEYWORD_ATTRIB_END);
      }
    }
    else
    {
      if ( strncmp(place_cur->getName(), NOTATION_REDUNDANT_PLACE_PREFIX, len) == 0 )
        fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
          DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_REDUNDANT_MARKED, DOT_KEYWORD_ATTRIB_END);
      else
      {
        if (cluster_cur->getType() == Cluster::TYPE_INPUT)
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN,
             DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_MARKED_INPUT, DOT_KEYWORD_ATTRIB_END);
        else
        if (cluster_cur->getType() == Cluster::TYPE_OUTPUT)
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
            DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_MARKED_OUTPUT, DOT_KEYWORD_ATTRIB_END);
        else
          fprintf(file, "    %s %s%s=\"%s\", %s%s;\n", place_var, DOT_KEYWORD_ATTRIB_BEGIN, 
            DOT_KEYWORD_LABEL, place_label, DOT_ATTRIB_PLACE_MARKED, DOT_KEYWORD_ATTRIB_END);
      }
    }
    delete [] place_var;
    delete [] place_label;
  }
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDotCluster(FILE* file, int cluster_index)
{
  Cluster* cluster_cur = (*clusters)[cluster_index];
  // open cluster
  fprintf(file, "\n");
  fprintf(file, "  %s %s_%d\n", DOT_KEYWORD_SUBGRAPH, DOT_KEYWORD_CLUSTER, cluster_index);
  fprintf(file, "  %s\n", DOT_KEYWORD_STATEMENT_BEGIN);
  fprintf(file, "    %s=%s;\n", DOT_KEYWORD_LABEL, cluster_cur->getName());
  fprintf(file, "    %s=%s;\n", DOT_KEYWORD_COLOR, DOT_COLOR_WHITE);
  // write cluster  
  writeDotClusterProducingArcs(file, cluster_index);
  writeDotClusterConsumingArcs(file, cluster_index);
  writeDotClusterReadArcs(file, cluster_index);
  writeDotClusterTransitions(file, cluster_index);
  writeDotClusterPlaces(file, cluster_index);
  // close cluster
  fprintf(file, "  %s\n", DOT_KEYWORD_STATEMENT_END);
}

//---------------------------------------------------------------------------
void GraphClusterDot::writeDot(const char* file_name, int legend)
{
  if (file_name)
  {
    FILE* file = NULL;
    if (strcmp(file_name, NAME_STDIO) == 0)
      file = stdout;
    else
    {
      if ((file = fopen(file_name, "w")) == NULL)
        Error("opening output file <%s>", file_name);
    }
    // name
    char* model_tmp = genDotModelLabel(model);
    fprintf(file, "%s %s\n", DOT_KEYWORD_DIGRAPH, model_tmp);
    delete [] model_tmp;
    // header
    fprintf(file, "%s\n", DOT_KEYWORD_STATEMENT_BEGIN);
    fprintf(file, "  %s=%s;\n", DOT_KEYWORD_CENTER, "1");
    fprintf(file, "  %s=%s;\n", DOT_KEYWORD_MARGIN, "0");
    // legend
    if (legend > 0)
      writeDotLegend(file);
    // clusters
    for (int cluster_index = 0; cluster_index < clusters->count(); cluster_index++)
      writeDotCluster(file, cluster_index);
    // end
    fprintf(file, "%s\n",DOT_KEYWORD_STATEMENT_END);
    if (file != stdout)
      fclose(file);
  }
}
