#ifndef _STG_GRAPH_PROTODE_H_
#define _STG_GRAPH_PROTODE_H_

#include <stdio.h>
#include "stg_object.h"
#include "stg_transition.h"
#include "stg_place.h"
#include "stg_protocol.h"
#include "stg_graph.h"

//---------------------------------------------------------------------------
class GraphProtode : public Graph
{
  public:
    GraphProtode(const char* _name = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    GraphProtode(const GraphProtode& graph_protode);
    ~GraphProtode();

    char* genSignalDummyName(Transition* transition);
    unsigned genTransitionDupIndex(Transition* transition, int dup);
    char* genPlaceDupName(Place* place, int dup);

    int getProtocolCount() const;
    Protocol* getProtocol(int protocol_index) const;
    Protocol* addProtocol(const char* protocol_name);
    bool addProtocol(Protocol* protocol);
    void addProtocols(ProtocolArray* protocols);
    void removeProtocol(Protocol* protocol);
    Protocol* findProtocol(const char* protocol_name) const;

    int getTrackerCount() const;
    Graph* getTracker(int tracker_index) const;
    Graph* addTracker(const char* tracker_name);
    bool addTracker(Graph* tracker);
    void addTrackers(GraphArray* trackers);
    void removeTracker(Graph* tracker);
    Graph* findTracker(const char* tracker_name) const;

    int getProtocolObjectCount(const char* object_name) const;
    void addProtocolsDevEnv();
    void addProtocolsSets(const char* signal_sets);
    void setProtocolsExtract(const char* protocol_names);
    void setProtocolsOptimise(const char* protocol_names, int optimisation_level);
    void setProtocolsWrite(const char* protocol_names);
    void calcProtocols();
    bool checkProtocols();
    void extractProtocols();
    void optimiseProtocols();
    void writeProtocolsAstg(const char* file_name, bool show_implicit);

  protected:
    Graph* interface;
    GraphArray* trackers;
    GraphHashSet* tracker_hashset;
    ProtocolArray* protocols;
    ProtocolHashSet* protocol_hashset;
    
    void clean();
    
    void exposeTransition(Graph* tracker, int transition_index, Place* place_from, Place* place_to);
    void exposeTransitions(Graph* tracker, Signal* signal, Place* place_low, Place* place_high);
    void controlTransition(Graph* tracker, int transition_index, Place* place_to);
    void controlTransitions(Graph* tracker, Signal* signal, Place* place_low, Place* place_high);
    void exposeSignal(Signal* signal);
    bool hasArbitration(Place* place, Transition* transition);
    void exposeArbitration(Place* place);

    void findPlaceDeepPostset(Place* place, int depth, TransitionArray* transition_set, PlaceArray* place_set);
    void findTransitionDeepPostset(Transition* transition, int depth, PlaceArray* place_set, TransitionArray* transition_set);
    void findPlaceDeepPreset(Place* place, int depth, TransitionArray* transition_set, PlaceArray* place_set);
    void findTransitionDeepPreset(Transition* transition, int depth, PlaceArray* place_set, TransitionArray* transition_set);

    void findBackwardNeighbourhood(Place* place, TransitionArray* transition_set);
    TransitionArray* calcBackwardNeighbourhood(Graph* tracker, Place* place);
    void findForwardNeighbourhood(Place* place, TransitionArray* transition_set);
    TransitionArray* calcForwardNeighbourhood(Graph* tracker, Place* place);
    bool isTrackerPlaceRedundant(Graph* tracker, Place* place);

    void optimiseProtocolChoice(Protocol* protocol);
    void optimiseProtocolLatency(Protocol* protocol);
    void optimiseProtocolSizeChain(Protocol* protocol);
    void optimiseProtocolSizeIndividual(Protocol* protocol);
    void optimiseProtocolMandatary(Protocol* protocol);

    Transition* findBestRollbackTransition(Place* place);
    void transformProtocolMarking(Protocol* protocol);
    
    void transformBouncerReadArc(Graph* tracker, Place* place_redundant, Transition* transition_read);
    void transformProtocolBouncer(Protocol* protocol);
    
    void transformTrackerRedundantPlace(Graph* tracker, Place* place_redundant);
    void transformProtocolTracker(Protocol* protocol);

    void writeProtocolsAstgSignals(FILE* file, Signal::Type type, const char* label);
    void writeProtocolsAstgInitialStates(FILE* file);
    void writeProtocolsAstgProducingArcs(FILE* file, bool show_implicit);
    void writeProtocolsAstgConsumingArcs(FILE* file, bool show_implicit);
    void writeProtocolsAstgReadArcs(FILE* file);
    void writeProtocolsAstgArcs(FILE* file, bool show_implicit);
    void writeProtocolsAstgMarking(FILE* file, bool show_implicit);
};

//---------------------------------------------------------------------------
inline int GraphProtode::getProtocolCount() const
{
  return protocols->count();
}

//---------------------------------------------------------------------------
inline Protocol* GraphProtode::getProtocol(int protocol_index) const
{
  return (*protocols)[protocol_index];
}

//---------------------------------------------------------------------------
inline int GraphProtode::getTrackerCount() const
{
  return trackers->count();
}

//---------------------------------------------------------------------------
inline Graph* GraphProtode::getTracker(int tracker_index) const
{
  return (*trackers)[tracker_index];
}

#endif
