#include "array.h"
#include "stg_graph.h"
#include "stg_graph_utils.h"

//---------------------------------------------------------------------------
void addConsumingArc(Place* place, Transition* transition)
{
   place->addPostset(transition);
   transition->addPreset(place);
}

//---------------------------------------------------------------------------
void addConsumingArcs(PlaceArray* places, Transition* transition, Place* place_exclude)
{
  int count = places->count();
  for (int place_index = 0; place_index < count; place_index++)
  {
    Place* place = (*places)[place_index];
    if (place != place_exclude)
      addConsumingArc(place, transition);
  }
}

//---------------------------------------------------------------------------
void addConsumingArcs(Place* place, TransitionArray* transitions, Transition* transition_exclude)
{
  int count = transitions->count();
  for (int transition_index = 0; transition_index < count; transition_index++)
  {
    Transition* transition = (*transitions)[transition_index];
    if (transition != transition_exclude)
      addConsumingArc(place, transition);
  }
}

//---------------------------------------------------------------------------
void removeConsumingArc(Place* place, Transition* transition)
{
  place->removePostset(transition);
  transition->removePreset(place);
}

//---------------------------------------------------------------------------
void removeConsumingArcs(PlaceArray* places, Transition* transition, Place* place_exclude)
{
  PlaceArray* places_tmp = new PlaceArray(*places);
  for (int place_index = 0; place_index < places_tmp->count(); place_index++)
  {
    Place* place = (*places_tmp)[place_index];
    if (place != place_exclude)
      removeConsumingArc(place, transition);
  }
  delete places_tmp;
}

//---------------------------------------------------------------------------
void removeConsumingArcs(Place* place, TransitionArray* transitions, Transition* transition_exclude)
{
  TransitionArray* transitions_tmp = new TransitionArray(*transitions);
  for (int transition_index = 0; transition_index < transitions_tmp->count(); transition_index++)
  {
    Transition* transition = (*transitions_tmp)[transition_index];
    if (transition != transition_exclude)
      removeConsumingArc(place, transition);
  }
  delete transitions_tmp;
}

//---------------------------------------------------------------------------
bool isConsumingArc(Place* place, Transition* transition)
{
  return ( (place->locatePostset(transition) >= 0)
        && (transition->locatePreset(place) >= 0) );
}

//---------------------------------------------------------------------------
void addProducingArc(Transition* transition, Place* place)
{
   transition->addPostset(place);
   place->addPreset(transition);
}

//---------------------------------------------------------------------------
void addProducingArcs(TransitionArray* transitions, Place* place, Transition* transition_exclude)
{
  int count = transitions->count();
  for (int transition_index = 0; transition_index < count; transition_index++)
  {
    Transition* transition = (*transitions)[transition_index];
    if (transition != transition_exclude)
      addProducingArc(transition, place);
  }
}

//---------------------------------------------------------------------------
void addProducingArcs(Transition* transition, PlaceArray* places, Place* place_exclude)
{
  int count = places->count();
  for (int place_index = 0; place_index < count; place_index++)
  {
    Place* place = (*places)[place_index];
    if (place != place_exclude)
      addProducingArc(transition, place);
  }
}

//---------------------------------------------------------------------------
void removeProducingArc(Transition* transition, Place* place)
{
  transition->removePostset(place);
  place->removePreset(transition);
}

//---------------------------------------------------------------------------
void removeProducingArcs(TransitionArray* transitions, Place* place, Transition* transition_exclude)
{
  TransitionArray* transitions_tmp = new TransitionArray(*transitions);
  for (int transition_index = 0; transition_index < transitions_tmp->count(); transition_index++)
  {
    Transition* transition = (*transitions_tmp)[transition_index];
    if (transition != transition_exclude)
      removeProducingArc(transition, place);
  }
  delete transitions_tmp;
}

//---------------------------------------------------------------------------
void removeProducingArcs(Transition* transition, PlaceArray* places, Place* place_exclude)
{
  PlaceArray* places_tmp = new PlaceArray(*places);
  for (int place_index = 0; place_index < places_tmp->count(); place_index++)
  {
    Place* place = (*places_tmp)[place_index];
    if (place != place_exclude)
      removeProducingArc(transition, place);
  }
  delete places_tmp;
}

//---------------------------------------------------------------------------
bool isProducingArc(Transition* transition, Place* place)
{
  return ( (transition->locatePostset(place) >= 0)
        && (place->locatePreset(transition) >= 0) );
}

//---------------------------------------------------------------------------
void addReadArc(Place* place, Transition* transition)
{
   place->addReadset(transition);
   transition->addReadset(place);
}

//---------------------------------------------------------------------------
void addReadArcs(PlaceArray* places, Transition* transition, Place* place_exclude)
{
  int count = places->count();
  for (int place_index = 0; place_index < count; place_index++)
  {
    Place* place = (*places)[place_index];
    if (place != place_exclude)
      addReadArc(place, transition);
  }
}

//---------------------------------------------------------------------------
void addReadArcs(Place* place, TransitionArray* transitions, Transition* transition_exclude)
{
  int count = transitions->count();
  for (int transition_index = 0; transition_index < count; transition_index++)
  {
    Transition* transition = (*transitions)[transition_index];
    if (transition != transition_exclude)
      addReadArc(place, transition);
  }
}

//---------------------------------------------------------------------------
void removeReadArc(Place* place, Transition* transition)
{
   place->removeReadset(transition);
   transition->removeReadset(place);
}

//---------------------------------------------------------------------------
void removeReadArcs(PlaceArray* places, Transition* transition, Place* place_exclude)
{
  PlaceArray* places_tmp = new PlaceArray(*places);
  for (int place_index = 0; place_index < places_tmp->count(); place_index++)
  {
    Place* place = (*places_tmp)[place_index];
    if (place != place_exclude)
      removeReadArc(place, transition);
  }
  delete places_tmp;
}

//---------------------------------------------------------------------------
void removeReadArcs(Place* place, TransitionArray* transitions, Transition* transition_exclude)
{
  TransitionArray* transitions_tmp = new TransitionArray(*transitions);
  for (int transition_index = 0; transition_index < transitions_tmp->count(); transition_index++)
  {
    Transition* transition = (*transitions_tmp)[transition_index];
    if (transition != transition_exclude)
      removeReadArc(place, transition);
  }
  delete transitions_tmp;
}

//---------------------------------------------------------------------------
bool isReadArc(Place* place, Transition* transition)
{
  return ( (place->locateReadset(transition) >= 0)
        && (transition->locateReadset(place) >= 0) );
}

//---------------------------------------------------------------------------
void swapTransitionsPreset(Transition* transition_1, Transition* transition_2)
{
  PlaceArray* preset_tmp = transition_1->getPreset();
  transition_1->setPreset(transition_2->getPreset());
  transition_2->setPreset(preset_tmp);
  for (int i = 0; i < transition_1->getPresetCount(); i++)
  {
    Place* place_cur = transition_1->getPreset(i);
    int position = place_cur->locatePostset(transition_2);
    place_cur->replacePostset(transition_1, position);
  }
  for (int i = 0; i < transition_2->getPresetCount(); i++)
  {
    Place* place_cur = transition_2->getPreset(i);
    int position = place_cur->locatePostset(transition_1);
    place_cur->replacePostset(transition_2, position);
  }
}

//---------------------------------------------------------------------------
void swapTransitionsPostset(Transition* transition_1, Transition* transition_2)
{
  PlaceArray* postset_tmp = transition_1->getPostset();
  transition_1->setPostset(transition_2->getPostset());
  transition_2->setPostset(postset_tmp);
  for (int i = 0; i < transition_1->getPostsetCount(); i++)
  {
    Place* place_cur = transition_1->getPostset(i);
    int position = place_cur->locatePreset(transition_2);
    place_cur->replacePreset(transition_1, position);
  }
  for (int i = 0; i < transition_2->getPostsetCount(); i++)
  {
    Place* place_cur = transition_2->getPostset(i);
    int position = place_cur->locatePreset(transition_1);
    place_cur->replacePreset(transition_2, position);
  }
}

//---------------------------------------------------------------------------
void swapTransitionsReadset(Transition* transition_1, Transition* transition_2)
{
  PlaceArray* readset_tmp = transition_1->getReadset();
  transition_1->setReadset(transition_2->getReadset());
  transition_2->setReadset(readset_tmp);
  for (int i = 0; i < transition_1->getReadsetCount(); i++)
  {
    Place* place_cur = transition_1->getReadset(i);
    int position = place_cur->locateReadset(transition_2);
    place_cur->replaceReadset(transition_1, position);
  }
  for (int i = 0; i < transition_2->getReadsetCount(); i++)
  {
    Place* place_cur = transition_2->getReadset(i);
    int position = place_cur->locateReadset(transition_1);
    place_cur->replaceReadset(transition_2, position);
  }
}
//---------------------------------------------------------------------------
void swapTransitions(Transition* transition_1, Transition* transition_2)
{
  swapTransitionsPreset(transition_1, transition_2);
  swapTransitionsPostset(transition_1, transition_2);
  swapTransitionsReadset(transition_1, transition_2);
}

//---------------------------------------------------------------------------
void swapPlacesPreset(Place* place_1, Place* place_2)
{
  TransitionArray* preset_tmp = place_1->getPreset();
  place_1->setPreset(place_2->getPreset());
  place_2->setPreset(preset_tmp);
  for (int i = 0; i < place_1->getPresetCount(); i++)
  {
    Transition* transition_cur = place_1->getPreset(i);
    int position = transition_cur->locatePostset(place_2);
    transition_cur->replacePostset(place_1, position);
  }
  for (int i = 0; i < place_2->getPresetCount(); i++)
  {
    Transition* transition_cur = place_2->getPreset(i);
    int position = transition_cur->locatePostset(place_1);
    transition_cur->replacePostset(place_2, position);
  }
}

//---------------------------------------------------------------------------
void swapPlacesPostset(Place* place_1, Place* place_2)
{
  TransitionArray* postset_tmp = place_1->getPostset();
  place_1->setPostset(place_2->getPostset());
  place_2->setPostset(postset_tmp);
  for (int i = 0; i < place_1->getPostsetCount(); i++)
  {
    Transition* transition_cur = place_1->getPostset(i);
    int position = transition_cur->locatePreset(place_2);
    transition_cur->replacePreset(place_1, position);
  }
  for (int i = 0; i < place_2->getPostsetCount(); i++)
  {
    Transition* transition_cur = place_2->getPostset(i);
    int position = transition_cur->locatePreset(place_1);
    transition_cur->replacePreset(place_2, position);
  }
}

//---------------------------------------------------------------------------
void swapPlacesReadset(Place* place_1, Place* place_2)
{
  TransitionArray* readset_tmp = place_1->getReadset();
  place_1->setReadset(place_2->getReadset());
  place_2->setReadset(readset_tmp);
  for (int i = 0; i < place_1->getReadsetCount(); i++)
  {
    Transition* transition_cur = place_1->getReadset(i);
    int position = transition_cur->locateReadset(place_2);
    transition_cur->replaceReadset(place_1, position);
  }
  for (int i = 0; i < place_2->getReadsetCount(); i++)
  {
    Transition* transition_cur = place_2->getReadset(i);
    int position = transition_cur->locateReadset(place_1);
    transition_cur->replaceReadset(place_2, position);
  }
}

//---------------------------------------------------------------------------
void swapPlaces(Place* place_1, Place* place_2)
{
  swapPlacesPreset(place_1, place_2);
  swapPlacesPostset(place_1, place_2);
  swapPlacesReadset(place_1, place_2);
}
