#ifndef _STG_OBJECT_H_
#define _STG_OBJECT_H_

#include <string.h>
#include "str_utils.h"
#include "hash.h"
#include "hashset.h"
#include "array.h"

//---------------------------------------------------------------------------
class Object;
struct ObjectHashName;
struct ObjectEqualName;
struct ObjectCompareName;

typedef Array<Object, ObjectCompareName> ObjectArray;
typedef HashSet<Object, ObjectHashName, ObjectEqualName> ObjectHashSet;

//---------------------------------------------------------------------------
typedef unsigned char Tag;
extern "C" const Tag TAG_BIT_USER_1;
extern "C" const Tag TAG_BIT_USER_2;
extern "C" const Tag TAG_BIT_USER_3;
extern "C" const Tag TAG_BIT_USER_4;
extern "C" const Tag TAG_UNDEFINED;
extern "C" const Tag TAG_PROCESSED;
extern "C" const Tag TAG_GENERATED;
extern "C" const Tag TAG_HIDDEN;

//---------------------------------------------------------------------------
class Object
{
  public:
    Object(const char* _name = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Object(const Object& object);
    ~Object();

    void setName(const char* _name);
    const char* getName() const;
    void setTag(Tag _tag);
    Tag getTag() const;
    void addTag(Tag _tag);
    void removeTag(Tag _tag);
    bool testTag(Tag _tag) const;
    bool inTag(Tag _tag) const;
    void setData(void* _data);
    void* getData() const;

  protected:
    char* name;
  	Tag tag;
    void* data;
};

//---------------------------------------------------------------------------
inline const char* Object::getName() const
{
  return name;
}

//---------------------------------------------------------------------------
inline void Object::setTag(Tag _tag)
{
  tag = _tag;
}

//---------------------------------------------------------------------------
inline Tag Object::getTag() const
{
  return tag;
}

//---------------------------------------------------------------------------
inline void Object::addTag(Tag _tag)
{
  tag |= _tag;
}

//---------------------------------------------------------------------------
inline void Object::removeTag(Tag _tag)
{
  tag &= ~_tag;
}

//---------------------------------------------------------------------------
inline bool Object::testTag(Tag _tag) const
{
  return ( (tag & _tag) == _tag );
}

//---------------------------------------------------------------------------
inline bool Object::inTag(Tag _tag) const
{
  return ( (tag & _tag) == tag );
}

//---------------------------------------------------------------------------
inline void Object::setData(void* _data)
{
  data = _data;
}

//---------------------------------------------------------------------------
inline void* Object::getData() const
{
  return data;
}

//---------------------------------------------------------------------------
struct ObjectHashName
{
	unsigned operator()(const Object* object) const
	{
		return hash( (unsigned char*)object->getName(), strlen(object->getName()), 0);
	}
};

//---------------------------------------------------------------------------
struct ObjectEqualName
{
  bool operator()(const Object* object_lhs, const Object* object_rhs) const
  {
    return ( strcmp(object_lhs->getName(), object_rhs->getName()) == 0 );
  }
};

//---------------------------------------------------------------------------
struct ObjectCompareName
{
  int operator()(const Object* object_lhs, const Object* object_rhs) const
  {
    return strcmp(object_lhs->getName(), object_rhs->getName());
  }
};

#endif
