#ifndef _STG_PLACE_H_
#define _STG_PLACE_H_

#include "array.h"
#include "hashset.h"
#include "stg_object.h"

class Place;
typedef Array<Place, ObjectCompareName> PlaceArray;
typedef HashSet<Place, ObjectHashName, ObjectEqualName> PlaceHashSet;

#include "stg_transition.h"

//---------------------------------------------------------------------------
class Place : public Object
{
  public:
    Place(const char* _name, int _marking = 0, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Place(const Place& place);
    ~Place();

    void setMarking(int _marking);
    int getMarking() const;
    void decMarking();
    void incMarking();
 
    bool isImplicit() const;
    bool isChoice() const;
    bool isMerge() const;
    
    int getPresetCount() const;
    void setPreset(TransitionArray* _preset);
    TransitionArray* getPreset() const;
    Transition* getPreset(int position) const;
    void addPreset(Transition* transition);
    void replacePreset(Transition* transition, int position);
    void removePreset(Transition* transition);
    int locatePreset(Transition* transition) const;
    
    int getPostsetCount() const;
    void setPostset(TransitionArray* _postset);
    TransitionArray* getPostset() const;
    Transition* getPostset(int position) const;
    void addPostset(Transition* transition);
    void replacePostset(Transition* transition, int position);
    void removePostset(Transition* transition);
    int locatePostset(Transition* transition) const;
    
    int getReadsetCount() const;
    void setReadset(TransitionArray* _readset);
    TransitionArray* getReadset() const;
    Transition* getReadset(int position) const;
    void addReadset(Transition* transition);
    void replaceReadset(Transition* transition, int position);
    void removeReadset(Transition* transition);
    int locateReadset(Transition* transition) const;

  protected:
    int marking;
    TransitionArray* preset;
    TransitionArray* postset;
    TransitionArray* readset;
};

//---------------------------------------------------------------------------
inline void Place::setMarking(int _marking)
{
  marking = _marking;
}

//---------------------------------------------------------------------------
inline int Place::getMarking() const
{
  return marking;
}

//---------------------------------------------------------------------------
inline void Place::decMarking()
{
  marking--;
}

//---------------------------------------------------------------------------
inline void Place::incMarking()
{
  marking++;
}

//---------------------------------------------------------------------------
inline bool Place::isImplicit() const
{
  return ( (preset->count() == 1) && (postset->count() == 1) && (readset->count() == 0) );
}

//---------------------------------------------------------------------------
inline bool Place::isChoice() const
{
  return ( postset->count() > 1 );
}

//---------------------------------------------------------------------------
inline bool Place::isMerge() const
{
  return ( preset->count() > 1 );
}

//---------------------------------------------------------------------------
inline int Place::getPresetCount() const
{
  return preset->count();
}

//---------------------------------------------------------------------------
inline void Place::setPreset(TransitionArray* _preset)
{
  preset = _preset;
}

//---------------------------------------------------------------------------
inline TransitionArray* Place::getPreset() const
{
  return preset;
}

//---------------------------------------------------------------------------
inline Transition* Place::getPreset(int position) const
{
  return (*preset)[position];
}

//---------------------------------------------------------------------------
inline void Place::addPreset(Transition* transition)
{
  preset->add(transition);
}

//---------------------------------------------------------------------------
inline void Place::replacePreset(Transition* transition, int position)
{
  (*preset)[position] = transition;
}

//---------------------------------------------------------------------------
inline void Place::removePreset(Transition* transition)
{
  preset->remove(transition, false);
}

//---------------------------------------------------------------------------
inline int Place::locatePreset(Transition* transition) const
{
  return preset->locate(transition);
}

//---------------------------------------------------------------------------
inline int Place::getPostsetCount() const
{
  return postset->count();
}

//---------------------------------------------------------------------------
inline void Place::setPostset(TransitionArray* _postset)
{
  postset = _postset;
}

//---------------------------------------------------------------------------
inline TransitionArray* Place::getPostset() const
{
  return postset;
}

//---------------------------------------------------------------------------
inline Transition* Place::getPostset(int position) const
{
  return (*postset)[position];
}

//---------------------------------------------------------------------------
inline void Place::addPostset(Transition* transition)
{
  postset->add(transition);
}

//---------------------------------------------------------------------------
inline void Place::replacePostset(Transition* transition, int position)
{
  (*postset)[position] = transition;
}

//---------------------------------------------------------------------------
inline void Place::removePostset(Transition* transition)
{
  postset->remove(transition, false);
}

//---------------------------------------------------------------------------
inline int Place::locatePostset(Transition* transition) const
{
  return postset->locate(transition);
}

//---------------------------------------------------------------------------
inline int Place::getReadsetCount() const
{
  return readset->count();
}

//---------------------------------------------------------------------------
inline void Place::setReadset(TransitionArray* _readset)
{
  readset = _readset;
}

//---------------------------------------------------------------------------
inline TransitionArray* Place::getReadset() const
{
  return readset;
}

//---------------------------------------------------------------------------
inline Transition* Place::getReadset(int position) const
{
  return (*readset)[position];
}

//---------------------------------------------------------------------------
inline void Place::addReadset(Transition* transition)
{
  readset->add(transition);
}

//---------------------------------------------------------------------------
inline void Place::replaceReadset(Transition* transition, int position)
{
  (*readset)[position] = transition;
}

//---------------------------------------------------------------------------
inline void Place::removeReadset(Transition* transition)
{
  readset->remove(transition, false);
}

//---------------------------------------------------------------------------
inline int Place::locateReadset(Transition* transition) const
{
  return readset->locate(transition);
}

#endif
