#include "stg_object.h"
#include "stg_signal.h"
#include "stg_transition.h"
#include "stg_place.h"
#include "stg_protocol.h"

//---------------------------------------------------------------------------
Protocol::Protocol(const char* _name, Tag _tag, void* _data)
: Object(_name, tag, _data), extract(false), optimisation_level(0), write(false)
{
  object_hashset = new ObjectHashSet();
  objects = new ObjectArray(2);
}

//---------------------------------------------------------------------------
Protocol::Protocol(const Protocol& protocol)
: Object(protocol), extract(protocol.extract), 
  optimisation_level(protocol.optimisation_level), write(protocol.write)
{
  object_hashset = new ObjectHashSet();
  objects = new ObjectArray(protocol.objects->count());
  for (int i = 0; i < protocol.objects->count(); i++)
    addObject( new Object(*(*protocol.objects)[i]) );
}

//---------------------------------------------------------------------------
Protocol::~Protocol()
{
  clean();
  delete object_hashset; object_hashset = NULL;
  delete objects; objects = NULL;
}

//---------------------------------------------------------------------------
void Protocol::clean()
{
  if (objects) 
    objects->clean(true);
}

//---------------------------------------------------------------------------
Object* Protocol::addObject(const char* object_name)
{
  Object* object_new = new Object(object_name);
  Object* object = object_hashset->find(object_new);
  if (object)
    delete object_new;
  else
  {
    object = object_new;
    objects->add(object);
    object_hashset->add(object);
  }
  return object;
}

//---------------------------------------------------------------------------
bool Protocol::addObject(Object* object)
{
  bool result = false;
  if (!object_hashset->find(object))
  {
    objects->add(object);
    object_hashset->add(object);
    result = true;
  }
  return result;
}

//---------------------------------------------------------------------------
void Protocol::addObjects(ObjectArray* objects)
{
  if (objects)
  for (int i = 0; i < objects->count(); i++)
  {
    Object* object_cur = (*objects)[i];
    if ( !addObject(object_cur) )
      delete object_cur;
  }
}

//---------------------------------------------------------------------------
void Protocol::removeObject(Object* object)
{
  object_hashset->remove(object);
  objects->remove(object, true);
}

//---------------------------------------------------------------------------
Object* Protocol::findObject(const char* object_name) const
{
  Object* object_mask = new Object(object_name);
  Object* object_cur = object_hashset->find(object_mask);
  delete object_mask;
  return object_cur;
}
