#ifndef _STG_PROTOCOL_H_
#define _STG_PROTOCOL_H_

#include "stg_object.h"

class Protocol;
typedef Array<Protocol, ObjectCompareName> ProtocolArray;
typedef HashSet<Protocol, ObjectHashName, ObjectEqualName> ProtocolHashSet;

//---------------------------------------------------------------------------
class Protocol : public Object
{
  public:
    Protocol(const char* _name = NULL, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Protocol(const Protocol& protocol);
    ~Protocol();

    void setExtract(bool _extract);
    bool getExtract() const;
    void setOptimisationLevel(int _optimisation_level);
    int getOptimisationLevel() const;
    void setWrite(bool _write);
    bool getWrite() const;
    
    int getObjectCount() const;
    Object* getObject(int object_index) const;
    Object* addObject(const char* object_name);
    bool addObject(Object* object);
    void addObjects(ObjectArray* objects);
    void removeObject(Object* object);
    Object* findObject(const char* object_name) const;

  protected:
    ObjectArray* objects;
    ObjectHashSet* object_hashset;

    bool extract;
    int optimisation_level;
    bool write;
    
    void clean();
};

//---------------------------------------------------------------------------
inline void Protocol::setExtract(bool _extract)
{
  extract = _extract;
}

//---------------------------------------------------------------------------
inline bool Protocol::getExtract() const
{
  return extract;
}

//---------------------------------------------------------------------------
inline void Protocol::setOptimisationLevel(int _optimisation_level)
{
  optimisation_level = _optimisation_level;
}

//---------------------------------------------------------------------------
inline int Protocol::getOptimisationLevel() const
{
  return optimisation_level;
}

//---------------------------------------------------------------------------
inline void Protocol::setWrite(bool _write)
{
  write = _write;
}

//---------------------------------------------------------------------------
inline bool Protocol::getWrite() const
{
  return write;
}

//---------------------------------------------------------------------------
inline int Protocol::getObjectCount() const
{
  return objects->count();
}

//---------------------------------------------------------------------------
inline Object* Protocol::getObject(int object_index) const
{
  return (*objects)[object_index];
}

#endif
