#ifndef _STG_SIGNAL_H_
#define _STG_SIGNAL_H_

#include "array.h"
#include "hashset.h"
#include "stg_object.h"

class Signal;
typedef Array<Signal, ObjectCompareName> SignalArray;
typedef HashSet<Signal, ObjectHashName, ObjectEqualName> SignalHashSet;

//---------------------------------------------------------------------------
class Signal : public Object
{
  public:
    enum Type 
    { 
      TYPE_UNDEFINED,  
      TYPE_INPUT, 
      TYPE_OUTPUT, 
      TYPE_INTERNAL, 
      TYPE_DUMMY 
    };
    enum State 
    { 
      STATE_UNDEFINED,  
      STATE_HIGH, 
      STATE_LOW 
    };

    Signal(const char* _name, Type _type = TYPE_UNDEFINED, State _state = STATE_UNDEFINED, Tag _tag = TAG_UNDEFINED, void* _data = NULL);
    Signal(const Signal& signal);
    ~Signal();

    void setType(Type _type);
    Type getType() const;

    void setState(State _state);
    State getState() const;

  protected:
    Type type;
    State state;
};

//---------------------------------------------------------------------------
inline void Signal::setType(Type _type)
{
  type = _type;
}

//---------------------------------------------------------------------------
inline Signal::Type Signal::getType() const
{
  return type;
}

//---------------------------------------------------------------------------
inline void Signal::setState(State _state)
{
  this->state = _state;
}

//---------------------------------------------------------------------------
inline Signal::State Signal::getState() const
{
  return state;
}

#endif
