#include <string.h>
#include "str_utils.h"
#include "messages.h"
#include "stg_object.h"
#include "stg_transition.h"

const char* TRANSITION_PATTERN_DUMMY = "%s";
const char* TRANSITION_PATTERN_SIMPLE= "%s%c";
const char* TRANSITION_PATTERN_INDEX = "%s/%d";

const char TRANSITION_SYMBOL_RISING  = '+';
const char TRANSITION_SYMBOL_FALLING = '-';
const char TRANSITION_SYMBOL_INDEX   = '/';

//---------------------------------------------------------------------------
Transition::Transition(const char* _name, Tag _tag, void* _data) 
: Object(_name, _tag, _data), signal_name(NULL)
{
  parseName();
  preset = new PlaceArray(1);
  postset =  new PlaceArray(1);
  readset =  new PlaceArray(1);
}
//---------------------------------------------------------------------------
Transition::Transition(const char* _signal_name, Polarity _polarity, unsigned _index, Tag _tag, void* _data)
: Object(NULL, _tag, _data), signal_name(NULL), polarity(_polarity), index(_index)
{
  delete [] signal_name;
  signal_name = namecopy(_signal_name);
  genName();
  preset = new PlaceArray(1); 
  postset =  new PlaceArray(1);
  readset =  new PlaceArray(1);
}

//---------------------------------------------------------------------------
Transition::Transition(const Transition& transition) 
: Object(transition), signal_name(NULL)
{
  parseName();
  preset = new PlaceArray(*transition.preset);
  postset =  new PlaceArray(*transition.postset);
  readset =  new PlaceArray(*transition.readset);
}

//---------------------------------------------------------------------------
Transition::~Transition()
{
  delete [] signal_name;
  delete preset; preset = NULL;
  delete postset; postset = NULL;
  delete readset; readset = NULL;
}

//---------------------------------------------------------------------------
void Transition::genName()
{
  delete [] name;
  name = NULL;
  
  if (signal_name)
  {
    char* name_new = NULL;
    switch (polarity)
    {
      case POLARITY_UNDEFINED:
        name_new = name2pattern(TRANSITION_PATTERN_DUMMY, signal_name);
        break;
      case POLARITY_RISING:
        name_new = name2pattern(TRANSITION_PATTERN_SIMPLE, signal_name, TRANSITION_SYMBOL_RISING);
        break;
      case POLARITY_FALLING:
        name_new = name2pattern(TRANSITION_PATTERN_SIMPLE, signal_name, TRANSITION_SYMBOL_FALLING);
        break;
    }
    if (index > 0)
    {
      name = name2pattern(TRANSITION_PATTERN_INDEX, name_new, index);
      delete [] name_new;
    }
    else
      name = name_new;
  }
}

//---------------------------------------------------------------------------
void Transition::parseName()
{
  delete [] signal_name;
  signal_name = NULL;
  polarity = POLARITY_UNDEFINED; 
  index = 0;
  if (name)
  {
    signal_name = namecopy(name);
    char* polarity_str = strchr(signal_name, TRANSITION_SYMBOL_RISING);
    if (polarity_str)
      polarity = POLARITY_RISING;
    else
    {    
      polarity_str = strchr(signal_name, TRANSITION_SYMBOL_FALLING);
      if (polarity_str)
        polarity = POLARITY_FALLING;
    }

    char* index_str = strchr(signal_name, TRANSITION_SYMBOL_INDEX);
    if (index_str)
    {
      index = atoi(&index_str[1]);
      index_str[0] = '\0';
    }

    if (polarity_str)
      polarity_str[0] = '\0';
  }
}
